"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const inquirer_1 = __importDefault(require("inquirer"));
const axios_1 = __importDefault(require("axios"));
const moment_1 = __importDefault(require("moment"));
const nunjucks_1 = require("nunjucks");
const convert_case_1 = require("../../utils/convert-case");
const utils_1 = require("./utils");
const nunjucksEnv = (0, nunjucks_1.configure)(__dirname, {
    autoescape: false,
    trimBlocks: true,
    lstripBlocks: true,
});
const typeMap = {
    'New feature': 'feature',
    'Bug fix': 'bugfix',
    'Documentation change': 'docs',
    Refactoring: 'refactor',
    'Component style change': 'style',
    Enhancement: 'enhancement',
    'Test cases': 'test',
    'Continuous integration': 'ci',
    'Typescript definition change': 'typescript',
    'Breaking change': 'attention',
};
const getRecords = (mr) => {
    const content = mr.body.replace(/\r\n/g, '\n');
    const records = [];
    const typeRule = new RegExp('##\\s+Types of changes.+?\\[[xX]]\\s+(.+?)\\n', 's');
    const typeString = content.match(typeRule)?.[1];
    const type = typeString && typeMap[typeString];
    const rule = new RegExp(
    // Table title
    '##\\s+Changelog\\n\\n' +
        // Table title
        '\\s*\\|(.+)\\|\\s*\\n' +
        // Alignment info
        '\\s*\\|(?:[-: ]+[-| :]*)\\|\\s*\\n' +
        // Table content
        '((?:\\s*\\|.*\\|\\s*(?:\\n|$))*)');
    const matchResult = content.match(rule);
    if (matchResult) {
        const titles = matchResult[1].split('|').map((item) => item.toLowerCase().trim());
        const lines = matchResult[2].split('\n').filter((value) => Boolean(value.trim()));
        for (const line of lines) {
            const items = line
                .split('|')
                .map((value) => value.trim())
                .filter((value) => Boolean(value.trim()));
            if (items.length > 0) {
                const data = titles.reduce((data, title, index) => {
                    switch (title) {
                        case 'type': {
                            if (items[index] && typeMap[items[index]]) {
                                data[title] = typeMap[items[index]];
                            }
                            break;
                        }
                        case 'component':
                            data[title] = (0, convert_case_1.toKebabCase)(items[index] ?? '');
                            break;
                        case 'related issues': {
                            const match = (items[index] ?? '').match(/#\d+/g);
                            if (match) {
                                data.issue = match.map((item) => item.slice(1));
                            }
                            break;
                        }
                        default:
                            data[title] = items[index];
                    }
                    return data;
                }, {
                    mrId: mr.number,
                    mrURL: mr.html_url,
                    type,
                });
                records.push(data);
            }
        }
    }
    return records;
};
const addAll = (data, changelog) => {
    if (!changelog[data.type]) {
        changelog[data.type] = [];
    }
    if (!data.component || data.component === 'common') {
        changelog[data.type].push(data.content);
    }
    else {
        changelog[data.type].push(`**${data.component}:** ${data.content}`);
    }
};
const addComponent = (data, changelog) => {
    if (!data.component || data.component === 'common') {
        return;
    }
    if (!changelog[data.component]) {
        changelog[data.component] = {};
    }
    if (!changelog[data.component][data.type]) {
        changelog[data.component][data.type] = [];
    }
    changelog[data.component][data.type].push(data.content);
};
const getEmitsFromChangelog = async (changelog) => {
    const allCN = {};
    const addEN = {};
    const componentCN = {};
    const componentEN = {};
    for (const item of changelog.list) {
        if (!(0, utils_1.isValidComponent)(item.component)) {
            // eslint-disable-next-line no-await-in-loop
            const answer = await inquirer_1.default.prompt({
                type: 'input',
                name: 'component',
                message: `The component name '${item.component}' is invalid, please input the new name.[${item.mrId}]`,
                validate(input) {
                    return (0, utils_1.isValidComponent)(input);
                },
            });
            item.component = answer.component;
        }
        if (!item.type) {
            // eslint-disable-next-line no-await-in-loop
            const answer = await inquirer_1.default.prompt({
                type: 'list',
                name: 'type',
                choices: ['feature', 'bugfix', 'enhancement', 'style', 'typescript', 'attention'],
                message: `Please select the type for '${item.component}'.[${item.mrId}]`,
            });
            item.type = answer.type;
        }
        const contentCN = `${item['changelog(cn)']} ([#${item.mrId}](${item.mrURL}))`;
        const contentEN = `${item['changelog(en)']} ([#${item.mrId}](${item.mrURL}))`;
        addAll({ ...item, content: contentCN }, allCN);
        addAll({ ...item, content: contentEN }, addEN);
        addComponent({ ...item, content: contentCN }, componentCN);
        addComponent({ ...item, content: contentEN }, componentEN);
    }
    const emits = [
        {
            filename: 'CHANGELOG.zh-CN.md',
            template: 'template/main.zh-CN.njk',
            data: { version: changelog.version, date: changelog.date, ...allCN },
        },
        {
            filename: 'CHANGELOG.md',
            template: 'template/main.en-US.njk',
            data: { version: changelog.version, date: changelog.date, ...addEN },
        },
    ];
    for (const component of Object.keys(componentCN)) {
        let filename = `components/${component}/CHANGELOG.zh-CN.md`;
        if (component === 'icon') {
            filename = `components/icon-component/CHANGELOG.zh-CN.md`;
        }
        emits.push({
            filename,
            template: 'template/main.zh-CN.njk',
            data: {
                version: changelog.version,
                date: changelog.date,
                ...componentCN[component],
            },
        });
    }
    for (const component of Object.keys(componentEN)) {
        let filename = `components/${component}/CHANGELOG.md`;
        if (component === 'icon') {
            filename = `components/icon-component/CHANGELOG.md`;
        }
        emits.push({
            filename,
            template: 'template/main.en-US.njk',
            data: {
                version: changelog.version,
                date: changelog.date,
                ...componentEN[component],
            },
        });
    }
    return emits;
};
const appendChangelog = async (emit) => {
    const { filename, template, data } = emit;
    const content = nunjucksEnv.render(template, data);
    try {
        await fs_extra_1.default.access(filename);
        const origin = await fs_extra_1.default.readFile(filename, 'utf-8');
        const match = origin.match(/^(```yaml\n.*?\n```\n\n)?(.*)$/s);
        if (match) {
            await fs_extra_1.default.writeFile(filename, `${match[1] ?? ''}${content}\n${match[2]}`);
        }
    }
    catch {
        // eslint-disable-next-line
        await fs_extra_1.default.writeFile(filename, '```yaml\nchangelog: true\n```\n\n' + content);
    }
};
const getLastVersion = (content) => {
    const match = content.match(/## (\d+\.\d+\.\d+(-beta\.\d+)?)/);
    return match?.[1];
};
const run = async () => {
    let version = '2.0.0';
    if (fs_extra_1.default.existsSync('package.json')) {
        try {
            const packageJson = await fs_extra_1.default.readFile('package.json', 'utf8');
            const packageData = JSON.parse(packageJson);
            if (packageData.version) {
                version = packageData.version;
            }
        }
        catch {
            // eslint-disable-next-line no-console
            console.log('read version from package.json has error');
        }
    }
    const answer = await inquirer_1.default.prompt({
        type: 'input',
        name: 'version',
        message: 'Please input the version',
        default: version,
        validate(input) {
            return /\d+\.\d+\.\d+(-beta\.\d+)?/.test(input);
        },
    });
    version = answer.version;
    const currentContent = fs_extra_1.default.readFileSync('CHANGELOG.zh-CN.md', 'utf8');
    const lastVersion = getLastVersion(currentContent);
    if (lastVersion && (version === lastVersion || (0, utils_1.compareVersion)(version, lastVersion) < 1)) {
        const answer = await inquirer_1.default.prompt({
            type: 'input',
            name: 'version',
            message: `This version is already existed or lower than last version, please reenter`,
            validate(input) {
                return /\d+\.\d+\.\d+(-beta\.\d+)?/.test(input) && input !== lastVersion;
            },
        });
        version = answer.version;
    }
    const res = await axios_1.default.get(`https://api.github.com/search/issues?accept=application/vnd.github.v3+json&q=repo:ent-framework/fe-ent-core+is:pr+is:merged+milestone:${version}`);
    if (res.status === 200) {
        const { data } = res;
        const changelog = {
            version,
            date: (0, moment_1.default)().format('YYYY-MM-DD'),
            list: [],
        };
        for (const item of data?.items ?? []) {
            const records = getRecords(item);
            changelog.list.push(...records);
        }
        if (changelog.list.length > 0) {
            const emits = await getEmitsFromChangelog(changelog);
            for (const item of emits) {
                // eslint-disable-next-line no-await-in-loop
                await appendChangelog(item);
            }
        }
        else {
            // eslint-disable-next-line no-console
            console.log('No update information found');
        }
    }
};
exports.default = run;
//# sourceMappingURL=index.js.map