"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const vue_docgen_api_1 = require("vue-docgen-api");
const convert_case_1 = require("../../utils/convert-case");
const slot_tag_handler_1 = require("../docgen/slot-tag-handler");
const propExtHandler_1 = __importDefault(require("../docgen/propExtHandler"));
const get_package_1 = require("../../utils/get-package");
const getComponentsFromTemplates = async () => {
    const templates = await (0, fast_glob_1.default)('components/**/TEMPLATE.md');
    const components = [];
    for (const item of templates) {
        const dirname = path_1.default.dirname(item);
        const source = await fs_extra_1.default.readFile(item, 'utf8');
        const matches = Array.from(source.matchAll(/%%API\((.+?)\)%%/g));
        matches.forEach((match) => {
            if (match[1]) {
                components.push(path_1.default.resolve(dirname, match[1]));
            }
        });
    }
    return components;
};
const isLanguageTag = (title) => {
    return ['zh', 'en'].includes(title);
};
const resolveComponent = (doc) => {
    return {
        name: (0, convert_case_1.toKebabCase)(`${doc.displayName}`),
        props: doc.props
            ?.map((descriptor) => {
            let description;
            if (descriptor.description) {
                description = {
                    zh: descriptor.description,
                    en: descriptor.description,
                };
            }
            else {
                description = Object.values(descriptor.tags ?? {}).reduce((pre, item) => {
                    item.forEach((tag) => {
                        if (isLanguageTag(tag.title)) {
                            pre[tag.title] = tag.description;
                        }
                    });
                    return pre;
                }, { zh: '', en: '' });
            }
            return {
                name: (0, convert_case_1.toKebabCase)(descriptor.name),
                type: descriptor.type?.name,
                description,
            };
        })
            .filter((item) => Boolean(item.description.en)) ?? [],
        events: doc.events
            ?.map((descriptor) => {
            let description;
            if (descriptor.description) {
                description = {
                    zh: descriptor.description,
                    en: descriptor.description,
                };
            }
            else {
                description = (descriptor.tags ?? []).reduce((pre, item) => {
                    if (isLanguageTag(item.title)) {
                        // @ts-ignore
                        pre[item.title] = item.content;
                    }
                    return pre;
                }, { zh: '', en: '' });
            }
            return {
                name: (0, convert_case_1.toKebabCase)(descriptor.name),
                description,
            };
        })
            .filter((item) => Boolean(item.description.en) && !item.name.startsWith('update:')) ?? [],
        slots: doc.slots
            ?.map((descriptor) => {
            let description;
            if (descriptor.description) {
                description = {
                    zh: descriptor.description,
                    en: descriptor.description,
                };
            }
            else {
                description = Object.values(descriptor.tags ?? {}).reduce((pre, item) => {
                    // @ts-ignore
                    if (isLanguageTag(item.title)) {
                        // @ts-ignore
                        pre[item.title] = item.content;
                    }
                    return pre;
                }, { zh: '', en: '' });
            }
            return {
                name: (0, convert_case_1.toKebabCase)(descriptor.name),
                description,
            };
        })
            .filter((item) => Boolean(item.description.en)) ?? [],
    };
};
const isValidType = (type) => {
    const types = type.split('|').map((item) => item.trim());
    for (const item of types) {
        if (item && !/^(number|string|boolean|array|object)$/.test(item)) {
            return false;
        }
    }
    return true;
};
const transformToVetur = (components) => {
    const tags = {};
    const attributes = {};
    for (const component of components) {
        const attrs = [];
        for (const item of component.events ?? []) {
            attrs.push(item.name);
            attributes[`${component.name}/${item.name}`] = {
                description: item.description.en,
            };
        }
        for (const item of component.props ?? []) {
            const attrName = item.name.replace(/^on-/, '');
            if (!attrs.includes(attrName)) {
                attrs.push(attrName);
                attributes[`${component.name}/${attrName}`] = {
                    description: item.description.en,
                    type: isValidType(item.type) ? item.type : undefined,
                };
            }
        }
        tags[component.name] = {
            attributes: attrs,
        };
    }
    return {
        tags,
        attributes,
    };
};
const transformToWebTypes = (components, { version }) => {
    const json = {
        $schema: 'https://raw.githubusercontent.com/JetBrains/web-types/master/schema/web-types.json',
        framework: 'vue',
        name: 'fe-ent-core',
        version,
        contributions: {
            html: {
                'types-syntax': 'typescript',
                'description-markup': 'markdown',
                tags: [],
            },
        },
    };
    for (const component of components) {
        const data = {
            name: component.name,
            attributes: component.props?.map((item) => ({
                name: item.name,
                description: item.description.en,
                value: {
                    type: item.type,
                    kind: 'expression',
                },
            })),
            events: component.events?.map((item) => ({
                name: item.name,
                description: item.description.en,
            })),
            // @ts-ignore
            slots: component.slots?.map((item) => ({
                name: item.name,
                description: item.description.en,
            })),
        };
        // @ts-ignore
        json.contributions.html.tags.push(data);
    }
    return json;
};
const jsongen = async () => {
    const packageData = await (0, get_package_1.getPackage)();
    const components = await getComponentsFromTemplates();
    const docs = [];
    let typographyBase;
    let datePickerBase;
    for (const item of components) {
        const componentDoc = await (0, vue_docgen_api_1.parse)(item, {
            addScriptHandlers: [propExtHandler_1.default, slot_tag_handler_1.slotTagHandler],
        });
        const doc = resolveComponent(componentDoc);
        if (/date-picker\/picker/.test(item)) {
            datePickerBase = doc;
            continue;
        }
        if (/typography\/base.tsx/.test(item)) {
            typographyBase = doc;
            continue;
        }
        if (/date-picker\/pickers\//.test(item)) {
            doc.props.push(...(datePickerBase?.props ?? []));
            doc.events.push(...(datePickerBase?.events ?? []));
            doc.slots.push(...(datePickerBase?.slots ?? []));
        }
        if (/typography\/(title|paragraph|text)/.test(item)) {
            doc.props.push(...(typographyBase?.props ?? []));
            doc.events.push(...(typographyBase?.events ?? []));
            doc.slots.push(...(typographyBase?.slots ?? []));
        }
        docs.push(doc);
    }
    await fs_extra_1.default.emptyDir(path_1.default.resolve(process.cwd(), 'json'));
    // @ts-ignore
    const { tags, attributes } = transformToVetur(docs);
    await fs_extra_1.default.writeFile(path_1.default.resolve(process.cwd(), 'json/vetur-tags.json'), JSON.stringify(tags, null, 2));
    await fs_extra_1.default.writeFile(path_1.default.resolve(process.cwd(), 'json/vetur-attributes.json'), JSON.stringify(attributes, null, 2));
    // @ts-ignore
    const web = transformToWebTypes(docs, { version: packageData.version });
    await fs_extra_1.default.writeFile(path_1.default.resolve(process.cwd(), 'json/web-types.json'), JSON.stringify(web, null, 2));
};
exports.default = jsongen;
//# sourceMappingURL=index.js.map